package com.zilber.boot.activiti.service;


import com.zilber.boot.activiti.dto.InstanceDTO;
import com.zilber.boot.activiti.dto.PageQueryDTO;
import com.zilber.boot.activiti.vo.InstanceProgressVO;
import com.zilber.boot.activiti.vo.InstanceVO;
import org.activiti.engine.RuntimeService;
import org.activiti.engine.TaskService;
import org.activiti.engine.impl.persistence.entity.EntityManager;
import org.activiti.engine.runtime.ProcessInstance;
import org.activiti.engine.runtime.ProcessInstanceQuery;
import org.activiti.engine.task.Task;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

@Service
public class InstanceService {

    @Autowired
    private RuntimeService runtimeService;

    @Autowired
    private TaskService taskService;

    public ProcessInstance getById(String processInstanceId) {
        return runtimeService.createProcessInstanceQuery()
                .processInstanceId(processInstanceId)
                .singleResult();
    }

    public List<InstanceVO> getInstancePage(PageQueryDTO dto) {
        ProcessInstanceQuery query = runtimeService.createProcessInstanceQuery();
        return InstanceVO.merge(query.listPage(dto.getPageNo() * dto.getPageSize(), dto.getPageSize()));
    }

    /**
     * 启动流程实例并完成第一个任务
     *
     * @param dto 启动流程实例必要参数
     */
    public void start(InstanceDTO dto) {
        Map<String, Object> variables = dto.getVariables();
        if (variables == null) {
            variables = new HashMap<>();
        }

        // 设置默认的流程变量
        variables.put("assignee0", dto.getCreateBy());
        //variables.put("procDefKey", dto.getProcDefKey());

        // 启动流程实例
        ProcessInstance processInstance = runtimeService.startProcessInstanceByKey(
                dto.getProcDefKey(), /*dto.getBusinessId() +*/ "", variables);

        // 执行第一个任务
        Task task = taskService.createTaskQuery().processInstanceId(processInstance.getId()).singleResult();
        if (task != null) {
            if (task.getAssignee() == null) {
                taskService.claim(task.getId(), dto.getCreateBy());
            }
            taskService.complete(task.getId());
        }

    }

    /**
     * 查询指定流程进度
     *
     * @param instanceId 流程实例ID
     * @return 进度列表
     */
    public List<InstanceProgressVO> getInstanceProgress(String instanceId) {
        String sql = "select\n" +
                "    t1.ACT_ID_ as actId,\n" +
                "    t1.TASK_ID_ as taskId,\n" +
                "    t1.ACT_NAME_ as actName,\n" +
                "    t1.ACT_TYPE_ as actType,\n" +
                "    t1.PROC_INST_ID_ as procInstId,\n" +
                "    t1.DURATION_ as duration,\n" +
                "    t1.ACT_INST_STATE_ as state,\n" +
                "    t1.START_TIME_ as startTime,\n" +
                "    t1.END_TIME_ as endTime,\n" +
                "    t1.ASSIGNEE_ as assignee,\n" +
                "    t2.MESSAGE_ as remark,\n" +
                "    t3.TEXT_ as operators\n" +
                "from ACT_HI_ACTINST as t1\n" +
                "         left join ACT_HI_COMMENT as t2 on t1.TASK_ID_ = t2.TASK_ID_\n" +
                "         left join ACT_HI_VARINST as t3 on t1.ID_ = t3.ACT_INST_ID_ and t3.NAME_ = ?2\n" +
                "where t1.PROC_INST_ID_ = ?1\n" +
                "  " +
                "and t1.ACT_TYPE_ = 'userTask'\n" +
                "order by t1.START_TIME_ desc, NOT ISNULL(t1.END_TIME_), t1" +
                ".END_TIME_ desc";

        /*Query nativeQuery = entityManager.createNativeQuery(sql);
        nativeQuery.setParameter(1, instanceId);
        nativeQuery.setParameter(2, CamundaConstants.OPERATORS);
        return nativeQuery
                .unwrap(NativeQueryImpl.class)
                .setResultTransformer(Transformers.aliasToBean(InstanceProgressVO.class))
                .list();*/
        return null;
    }

    /**
     * 挂起流程实例
     *
     * @param id 流程实例id
     */
    public void suspend(String id) {
        runtimeService.suspendProcessInstanceById(id);
    }

    /**
     * 激活流程实例
     *
     * @param id 流程实例id
     */
    public void resume(String id) {
        runtimeService.activateProcessInstanceById(id);
    }

    /**
     * 结束流程实例（审批失败）
     *
     * @param id     流程实例id
     * @param reason 失败原因
     */
    public void failEndById(String id, String reason) {
        ProcessInstance processInstance = runtimeService.createProcessInstanceQuery()
                .processInstanceId(id).singleResult();
        if (processInstance != null) {
            // 审批失败，状态为 INTERNALLY_TERMINATED
            runtimeService.deleteProcessInstance(processInstance.getId(), reason);
        }
    }

    /**
     * 取消流程实例
     *
     * @param id 流程实例id
     */
    public void cancelById(String id) {
        ProcessInstance processInstance = runtimeService.createProcessInstanceQuery()
                .processInstanceId(id).singleResult();
        if (processInstance != null) {
            // 手动取消，状态为 EXTERNALLY_TERMINATED
            runtimeService.deleteProcessInstance(processInstance.getId(), "手动取消");
        }
    }

    /**
     * 获取流程变量
     *
     * @param id 流程实例id
     * @return 流程变量map
     */
    public Map<String, Object> getVariables(String id) {
        return runtimeService.getVariables(id);
    }

}
